{% extends "base.html" %}
{% block content %}
<h1 class="display-4">{{page.meta.title}}</h1>

{% if page.meta.description %}
<p>{{page.meta.description}}</p>
{% endif %}

<p class="h5">
{% for name, category_page in page.categories.items() %}
<a class="badge badge-pill badge-primary" href="{{url_for(category_page)}}">{{name}}
  <span class="badge badge-light">{{category_page.meta.pages|length}}</span>
</a>
{% endfor %}
</p>

{% endblock %}
