{% extends "base.html" %}
{% block content %}
<h1>{{page.meta.title}}</h1>

<table class="table table-striped table-responsive">
  <thead>
    <tr>
      <th>Date</th>
      <th>Page</th>
      <th>Title</th>
    </tr>
  </thead>
  <tbody>
  {% if page.meta.parent %}
  {% with parent = page.meta.parent %}
  <tr>
    <td><a href="{{url_for(parent)}}">{{parent.meta.date.strftime("%Y-%m-%d")}}</a></td>
    <td><a href="{{url_for(parent)}}">.. {% if parent.node.name %}({{parent.node.name}}){% endif %}</a></td>
    <td><a href="{{url_for(parent)}}">{{parent.meta.title}}</a></td>
  </tr>
  {% endwith %}
  {% endif %}
  {% for p in page.subdirs|arrange("title") %}
  <tr>
    <td><a href="{{url_for(p)}}">{{p.meta.date.strftime("%Y-%m-%d")}}</a></td>
    <td><a href="{{url_for(p)}}">{{p.node.name}}/</a></td>
    <td><a href="{{url_for(p)}}">{{p.meta.title}}</a></td>
  </tr>
  {% endfor %}
  {% for p in page.meta.pages|arrange("title") %}
  <tr>
    <td><a href="{{url_for(p)}}">{{p.meta.date.strftime("%Y-%m-%d")}}</a></td>
    <td><a href="{{url_for(p)}}">{{p.node.name}}</a></td>
    <td><a href="{{url_for(p)}}">{{p.meta.title}}</a></td>
  </tr>
  {% endfor %}
  </tbody>
</table>

{% endblock %}

