{% extends "base.html" %}

{% block front_matter %}
---
pages: "^\\d+/"
syndication:
  title: "Example blog feed"
{% endblock %}

{% import 'lib/blog.html' as blog with context %}

{% block content %}

<h1 class="display-4">Last 10 blog posts</h1>

<p>This the example front page for the blog.</p>

{{blog.pages(limit=10)}}

{% endblock %}
