/* Bootstrap does not seem to have a good default for blockquote */
blockquote {
    margin-left: 2em;
    margin-right: 2em;
    text-align: justify;
    font-style: italic;
    font-family: serif;
}

ul.taglist {
    padding-left: 0;
}

ul.taglist li {
}

div.codehilite {
    background-color: #f6f7f8;
    padding: 1em;
    border: 1pt dashed black;
    margin-bottom: 1rem;
    overflow: auto;
}

div.codehilite pre {
    margin: 0;
    overflow: visible;
}
